# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.v1.datamodel import pipeline as pipelinemodel
from solum.api.controllers.v1 import pipeline
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.pipeline_handler.PipelineHandler')
class TestPipelineController(base.BaseTestCase):
    def setUp(self):
        super(TestPipelineController, self).setUp()
        objects.load()

    def test_pipeline_get(self, PipelineHandler, resp_mock,
                          request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = PipelineHandler.return_value.get
        fake_pipeline = fakes.FakePipeline()
        hand_get.return_value = fake_pipeline
        resp = pipeline.PipelineController('test_id').get()
        self.assertIsNotNone(fake_pipeline)
        self.assertEqual(fake_pipeline.name, resp['result'].name)
        self.assertEqual(fake_pipeline.project_id, resp['result'].project_id)
        self.assertEqual(fake_pipeline.uuid, resp['result'].uuid)
        self.assertEqual(fake_pipeline.user_id, resp['result'].user_id)
        hand_get.assert_called_with('test_id')
        self.assertEqual(200, resp_mock.status)

    def test_pipeline_get_not_found(self, PipelineHandler,
                                    resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = PipelineHandler.return_value.get
        hand_get.side_effect = exception.ResourceNotFound(
            name='pipeline', pipeline_id='test_id')
        pipeline.PipelineController('test_id').get()
        hand_get.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_pipeline_put_none(self, PipelineHandler, resp_mock,
                               request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = None
        request_mock.content_type = 'application/json'
        hand_put = PipelineHandler.return_value.put
        hand_put.return_value = fakes.FakePipeline()
        pipeline.PipelineController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_pipeline_put_not_found(self, PipelineHandler,
                                    resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = PipelineHandler.return_value.update
        hand_update.side_effect = exception.ResourceNotFound(
            name='pipeline', pipeline_id='test_id')
        pipeline.PipelineController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(404, resp_mock.status)

    def test_pipeline_put_ok(self, PipelineHandler, resp_mock,
                             request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_update = PipelineHandler.return_value.update
        hand_update.return_value = fakes.FakePipeline()
        pipeline.PipelineController('test_id').put()
        hand_update.assert_called_with('test_id', json_update)
        self.assertEqual(200, resp_mock.status)

    def test_pipeline_delete_not_found(self, PipelineHandler,
                                       resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = PipelineHandler.return_value.delete
        hand_delete.side_effect = exception.ResourceNotFound(
            name='pipeline', pipeline_id='test_id')
        pipeline.PipelineController('test_id').delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_pipeline_delete_ok(self, PipelineHandler,
                                resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = PipelineHandler.return_value.delete
        hand_delete.return_value = None
        pipeline.PipelineController('test_id').delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(204, resp_mock.status)


class TestPipelineAsDict(base.BaseTestCase):

    scenarios = [
        ('none', dict(data=None)),
        ('one', dict(data={'name': 'foo'})),
        ('full', dict(data={'uri': 'http://example.com/v1/pipelines/x1',
                            'name': 'Example-pipeline',
                            'type': 'pipeline',
                            'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                            'user_id': '55f41cf46df74320b9486a35f5d28a11'}))
    ]

    def test_as_dict(self):
        objects.load()
        if self.data is None:
            s = pipelinemodel.Pipeline()
            self.data = {}
        else:
            s = pipelinemodel.Pipeline(**self.data)
        if 'uri' in self.data:
            del self.data['uri']
        if 'type' in self.data:
            del self.data['type']

        self.assertEqual(self.data, s.as_dict(objects.registry.Pipeline))


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.pipeline_handler.PipelineHandler')
class TestPipelinesController(base.BaseTestCase):
    def setUp(self):
        super(TestPipelinesController, self).setUp()
        objects.load()

    def test_pipelines_get_all(self, PipelineHandler,
                               resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = PipelineHandler.return_value.get_all
        fake_pipeline = fakes.FakePipeline()
        hand_get.return_value = [fake_pipeline]
        resp = pipeline.PipelinesController().get_all()
        self.assertEqual(fake_pipeline.name, resp['result'][0].name)
        self.assertEqual(fake_pipeline.project_id,
                         resp['result'][0].project_id)
        self.assertEqual(fake_pipeline.uuid, resp['result'][0].uuid)
        self.assertEqual(fake_pipeline.user_id, resp['result'][0].user_id)
        hand_get.assert_called_with()
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(resp)

    @mock.patch('solum.objects.registry.Plan')
    def test_pipelines_post(self, mock_Plan, PipelineHandler,
                            resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo',
                       'plan_uri': 'http://test_url:8080/test/911'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        request_mock.security_context = None
        mock_Plan.get_by_uuid.return_value = fakes.FakePlan()

        hand_create = PipelineHandler.return_value.create
        hand_create.return_value = fakes.FakePipeline()
        pipeline.PipelinesController().post()
        hand_create.assert_called_with({'name': 'foo',
                                        'plan_id': 8})
        mock_Plan.get_by_uuid.assert_called_with(None, '911')
        self.assertEqual(201, resp_mock.status)

    def test_pipelines_post_no_plan(self, PipelineHandler, resp_mock,
                                    request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        hand_create = PipelineHandler.return_value.create
        hand_create.return_value = fakes.FakePipeline()
        pipeline.PipelinesController().post()
        ret_val = pipeline.PipelinesController().post()
        faultstring = str(ret_val['faultstring'])
        self.assertIn('The plan was not given or could not be found',
                      faultstring)
        self.assertEqual(400, resp_mock.status)

    def test_pipelines_post_not_hosted(self, PipelineHandler, resp_mock,
                                       request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo',
                       'plan_uri': 'http://example.com/a.git'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        hand_create = PipelineHandler.return_value.create
        hand_create.return_value = fakes.FakePipeline()
        ret_val = pipeline.PipelinesController().post()
        faultstring = str(ret_val['faultstring'])
        self.assertIn('The plan was not hosted in solum', faultstring)
        self.assertEqual(400, resp_mock.status)

    def test_pipelines_post_nodata(self, PipelineHandler,
                                   resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        hand_create = PipelineHandler.return_value.create
        hand_create.return_value = fakes.FakePipeline()
        ret_val = pipeline.PipelinesController().post()
        faultstring = str(ret_val['faultstring'])
        self.assertEqual("Missing argument: \"data\"", faultstring)
        self.assertEqual(400, resp_mock.status)



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_objects
----------------------------------

Tests for the sqlalchemy solum 'objects' implementation
"""

import datetime

import testtools
from testtools import matchers

from solum.common import exception
from solum import objects
from solum.tests import base as tests
from solum.tests import utils

from oslo_utils import uuidutils


class TestObjectsSqlalchemy(tests.BaseTestCase):
    def setUp(self):
        super(tests.BaseTestCase, self).setUp()
        self.ctx = utils.dummy_context()
        self.useFixture(utils.Database())

    def test_objects_reloadable(self):
        self.assertIsNotNone(objects.registry.Component)

        objects.registry.clear()

        objects.registry.Component

        objects.load()

        self.assertIsNotNone(objects.registry.Component)

    def test_object_creatable(self):
        component = objects.registry.Component()
        self.assertIsNotNone(component)
        self.assertIsNone(component.id)

    # (claytonc) Depends on https://bugs.launchpad.net/oslo/+bug/1256954
    # for python 3.3
    # def test_object_duplicate_create(self):
    #     component = objects.registry.Component()
    #     component.create(self.ctx)
    #     component2 = objects.registry.Component()
    #     component2.id = component.id
    #     with testtools.ExpectedException(db_exc.DBError):
    #         component2.create(self.ctx)

    def test_object_raises_not_found(self):
        with testtools.ExpectedException(exception.ResourceNotFound):
            objects.registry.Component.get_by_id(None, 10000)

    def test_object_persist_and_retrieve(self):
        component = objects.registry.Component()
        self.assertIsNotNone(component)
        component.uuid = uuidutils.generate_uuid()
        component.name = 'abc'
        component.description = '1-2-3-4'
        component.plan_id = 1
        component.create(self.ctx)
        self.assertIsNotNone(component.id)

        component2 = objects.registry.Component.get_by_id(None, component.id)
        self.assertIsNotNone(component2)
        self.assertEqual(component.id, component2.id)
        self.assertEqual(component.uuid, component2.uuid)
        self.assertEqual(component.name, component2.name)
        self.assertEqual(component.description, component2.description)

        # visible via direct query
        dsession = utils.get_dummy_session()
        query = dsession.query(component.__class__).filter_by(id=component.id)
        component3 = query.first()
        self.assertIsNotNone(component3)
        self.assertEqual(component3.id, component3.id)

        # visible via get_all
        components = objects.registry.ComponentList.get_all(None)
        exists = [item for item in components if item.id == component.id]
        self.assertGreater(len(exists), 0)

    def test_object_mutate(self):
        begin = datetime.datetime.utcnow()

        component = objects.registry.Component()
        self.assertIsNotNone(component)
        component.uuid = uuidutils.generate_uuid()
        component.plan_id = 1
        component.create(self.ctx)

        self.assertIsNotNone(component.id)
        self.assertThat(component.created_at, matchers.GreaterThan(begin))
        self.assertIsNone(component.updated_at)

        next_time = datetime.datetime.utcnow()

        component.save(self.ctx)

        self.assertThat(next_time, matchers.GreaterThan(component.created_at))



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import userlog
from solum.api.handlers import userlog_handler
from solum.common import exception


class UserlogsController(rest.RestController):
    """Manages operations on the Userlogs collection."""

    def __init__(self, resource_id):
        super(UserlogsController, self).__init__()
        self._resource_id = resource_id

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose([userlog.Userlog])
    def get_all(self):
        """Return all Userlogs, based on the query provided."""
        handler = userlog_handler.UserlogHandler(
            pecan.request.security_context)
        ulogs = handler.get_all_by_id(self._resource_id)
        host_url = pecan.request.application_url.rstrip('/')
        return [userlog.Userlog.from_db_model(ulog, host_url)
                for ulog in ulogs]



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from wsme import types as wtypes


class Userlog(wtypes.Base):

    resource_uuid = wtypes.text
    resource_type = wtypes.text
    created_at = wtypes.text
    location = wtypes.text
    strategy = wtypes.text
    strategy_info = wtypes.text

    def __init__(self, **kwargs):
        super(Userlog, self).__init__(**kwargs)

    @classmethod
    def from_db_model(cls, m, host_url):
        return cls(
            resource_uuid=m.resource_uuid,
            resource_type=m.resource_type,
            created_at=str(m.created_at),
            location=m.location,
            strategy=m.strategy,
            strategy_info=m.strategy_info,
            )



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""empty message

Revision ID: 2e825c2c80c7
Revises: e5f2e8af3b8
Create Date: 2016-01-04 21:58:38.225814

"""
from alembic import op
import sqlalchemy as sa

from solum.objects.sqlalchemy import models

# revision identifiers, used by Alembic.
revision = '2e825c2c80c7'
down_revision = 'e5f2e8af3b8'


def upgrade():
    op.add_column('app',
                  sa.Column('raw_content', models.YAMLEncodedDict(2048)))


def downgrade():
    op.drop_column('app', 'raw_content')



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from wsme import types as wtypes

from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class ParameterDefinitionLink(common_types.Link):
    """CAMP v1.1 ParameterLink attribute type."""

    required = bool
    """Indicates whether the parameter referenced by this Link is required."""

    default_value = wtypes.Base
    """Default value for the parameter referenced by this Link."""


class ParameterDefinitions(api_types.Base):
    """CAMP v1.1 parameter_definitions resource model."""

    parameter_definition_links = [ParameterDefinitionLink]
    """The value of the parameter_definition_links attribute is an array of
       extended Link elements that reference parameter_definition resources.
     """

    def __init__(self, **kwds):
        super(ParameterDefinitions, self).__init__(**kwds)

    def fix_uris(self, host_url):
        """Update URIs to reflect a host URL."""

        ret_val = copy.deepcopy(self)
        ret_val.uri = uris.PARAM_DEFS_URI_STR % (host_url, ret_val.uri)
        for pd_link in ret_val.parameter_definition_links:
            pd_link.href = uris.PARAM_DEF_URI_STR % (host_url, pd_link.href)

        return ret_val



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from solum.objects import registry
from solum.objects.sqlalchemy import extension
from solum.tests import base
from solum.tests import utils


class TestExtension(base.BaseTestCase):
    def setUp(self):
        super(TestExtension, self).setUp()
        self.db = self.useFixture(utils.Database())
        self.ctx = utils.dummy_context()

        self.data = [{'uuid': 'test-uuid-42',
                      'project_id': self.ctx.project_id,
                      'user_id': '55f41cf46df74320b9486a35f5d28a11',
                      'name': 'logstash',
                      'version': '2.13',
                      'description': 'This logstash extension provides a tool'
                                     ' for managing your application events'
                                     ' and logs.',
                      'documentation': 'http://example.com/docs/ext/logstash',
                      }]
        utils.create_models_from_data(extension.Extension, self.data, self.ctx)

    def test_objects_registered(self):
        self.assertTrue(registry.Extension)
        self.assertTrue(registry.ExtensionList)

    def test_get_all(self):
        lst = extension.ExtensionList()
        self.assertIsNotNone(lst)
        self.assertEqual(1, len(lst.get_all(self.ctx)))

    def test_check_data_by_id(self):
        e = extension.Extension().get_by_id(self.ctx, self.data[0]['id'])
        self.assertIsNotNone(e)
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(e, key))

    def test_check_data_by_uuid(self):
        e = extension.Extension().get_by_uuid(self.ctx, self.data[0]['uuid'])
        self.assertIsNotNone(e)
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(e, key))



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import cfg

from solum.tests import base
from solum.tests import fakes
from solum.tests import utils
import solum.uploaders.swift as uploader


class SwiftUploadTest(base.BaseTestCase):
    def setUp(self):
        super(SwiftUploadTest, self).setUp()

    @mock.patch('builtins.open')
    @mock.patch('solum.uploaders.swift.SwiftUpload._upload')
    @mock.patch('solum.uploaders.common.UploaderBase.transform_jsonlog')
    @mock.patch('solum.uploaders.common.UploaderBase.write_userlog_row')
    def test_upload_on_assembly_delete(self, mock_write_row, mock_trans_jlog,
                                       mock_upload, mock_open):
        ctxt = utils.dummy_context()
        orig_path = "original path"
        assembly = fakes.FakeAssembly()
        build_id = "5678"
        container = 'fake-container'
        cfg.CONF.worker.log_upload_swift_container = container
        stage = "fakestage"

        swiftupload = uploader.SwiftUpload(ctxt, orig_path,
                                           assembly, build_id,
                                           stage)

        rs_before_delete = swiftupload.resource
        name_before = swiftupload.resource.name
        uuid_before = swiftupload.resource.uuid

        # Delete the assembly object before calling upload_log()
        del assembly

        swiftupload.upload_log()

        self.assertEqual(name_before, swiftupload.resource.name)
        self.assertEqual(uuid_before, swiftupload.resource.uuid)

        # Asserts
        swift_info = {'container': container}

        filename = "%s-%s/%s-%s.log" % (rs_before_delete.name,
                                        build_id,
                                        stage, build_id)

        self.assertTrue(mock_trans_jlog.called)
        tansf_path = orig_path + '.tf'
        mock_upload.assert_called_once_with(container, filename, tansf_path)
        mock_write_row.assert_called_once_with(filename, swift_info)

    @mock.patch('builtins.open')
    @mock.patch('solum.common.solum_swiftclient.SwiftClient.upload')
    @mock.patch('solum.uploaders.common.UploaderBase.transform_jsonlog')
    @mock.patch('solum.uploaders.common.UploaderBase.write_userlog_row')
    def test_upload(self, mock_write_row, mock_trans_jlog, mock_swift,
                    mock_open):
        ctxt = utils.dummy_context()
        orig_path = "original path"
        assembly = fakes.FakeAssembly()
        build_id = "5678"
        container = 'fake-container'
        cfg.CONF.worker.log_upload_swift_container = container
        stage = "fakestage"

        swiftupload = uploader.SwiftUpload(ctxt, orig_path,
                                           assembly, build_id,
                                           stage)
        resource = swiftupload.resource

        swiftupload.upload_log()

        swift_info = {'container': container}

        filename = "%s-%s/%s-%s.log" % (resource.name,
                                        build_id,
                                        stage, build_id)

        self.assertTrue(mock_trans_jlog.called)
        tansf_path = orig_path + '.tf'
        mock_swift.assert_called_once_with(tansf_path,
                                           container,
                                           filename)
        mock_write_row.assert_called_once_with(filename, swift_info)



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.tests import base
from solum.tests import fakes
from solum.tests import utils
import solum.uploaders.local as uploader


class LocalStorageTest(base.BaseTestCase):
    def setUp(self):
        super(LocalStorageTest, self).setUp()

    def test_upload(self):
        ctxt = utils.dummy_context()
        orig_path = "original path"
        assembly = fakes.FakeAssembly()
        build_id = "5678"
        localstorage = uploader.LocalStorage(ctxt, orig_path,
                                             assembly, build_id,
                                             "fakestage")
        localstorage.write_userlog_row = mock.MagicMock()
        localstorage.upload_log()

        localstorage.write_userlog_row.assert_called_once_with(orig_path)



# Copyright 2014 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sqlalchemy as sa

from solum.objects import service as abstract
from solum.objects.sqlalchemy import models as sql


class Service(sql.Base, abstract.Service):
    """Represent a service in sqlalchemy."""

    __resource__ = 'services'
    __tablename__ = 'service'
    __table_args__ = sql.table_args()

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)
    uuid = sa.Column(sa.String(36), nullable=False)
    name = sa.Column(sa.String(100))
    description = sa.Column(sa.String(255))
    project_id = sa.Column(sa.String(36))
    user_id = sa.Column(sa.String(36))
    service_type = sa.Column(sa.String(100))
    read_only = sa.Column(sa.Boolean, default=False)
    tags = sa.Column(sa.Text)

    def _non_updatable_fields(self):
        return set(('uuid', 'id', 'project_id'))


class ServiceList(abstract.ServiceList):
    """Represent a list of services in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        return ServiceList(sql.model_query(context, Service))



# Copyright 2015 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Base LP handler for building apps"""

import errno
import io
import json
import logging
import os
import random
import string
import time

import docker
from docker import errors
from oslo_config import cfg
from oslo_log import log as os_logging
from requests.packages.urllib3 import exceptions as req_exp

from solum.common import exception as exc
from solum.common import solum_swiftclient
from solum.uploaders import tenant_logger
from solum.worker.app_handlers import utils

from swiftclient import exceptions as swiftexp


LOG = os_logging.getLogger(__name__)

cfg.CONF.import_opt('task_log_dir', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('docker_daemon_url', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('docker_build_timeout', 'solum.worker.config',
                    group='worker')
cfg.CONF.import_opt('container_mem_limit', 'solum.worker.config',
                    group='worker')
log_dir = cfg.CONF.worker.task_log_dir
docker_daemon_url = cfg.CONF.worker.docker_daemon_url
build_timeout = cfg.CONF.worker.docker_build_timeout
mem_limit = cfg.CONF.worker.container_mem_limit

MAX_GIT_CLONE_RETRY = 5
GIT_CLONE_TIMEOUT = 900  # 15 minutes
cloner_gid = os.getgid()


class BaseHandler(object):

    def __init__(self, context, assembly, image_storage):
        self.context = context
        self.assembly = assembly
        self.image_storage = image_storage
        self._docker = None
        self.docker_cmd_uid = cfg.CONF.run_container_cmd_as
        self.cloner_image = None
        self.images = list()
        self.containers = list()
        self.work_dir = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.close()

    @property
    def docker(self):
        if self._docker is None:
            self._docker = docker.APIClient(base_url=docker_daemon_url)
        return self._docker

    def _get_tenant_logger(self, stage):
        return tenant_logger.TenantLogger(self.context,
                                          self.assembly, log_dir, stage)

    def close(self):
        for ct in self.containers:
            if ct:
                try:
                    self.docker.remove_container(container=ct.get('Id'))
                except (errors.DockerException, errors.APIError) as e:
                    LOG.warning('Failed to remove container %s, %s' %
                                (ct.get('Id'), str(e)))

        for img in self.images:
            try:
                self.docker.remove_image(image=img, force=True)
            except (errors.DockerException, errors.APIError) as e:
                LOG.warning('Failed to remove docker image %s, %s' %
                            (img, str(e)))

        if self.work_dir:
            self._remove_cloned_repo(self.work_dir)
            try:
                utils.rm_tree(self.work_dir)
            except OSError as e:
                if e.errno != errno.ENOENT:
                    LOG.critical('critical: cannot remove dir %s,'
                                 ' disk may be full.' % self.work_dir)

        if self.cloner_image:
            try:
                self.docker.remove_image(image=self.cloner_image, force=True)
            except (errors.DockerException, errors.APIError) as e:
                LOG.error('Error in removing docker image %s, %s' %
                          (self.cloner_image, str(e)))

    def _validate_pub_repo(self, repo_url):
        pass

    @utils.retry
    def _remove_cloned_repo(self, destination):
        if not os.path.exists(destination):
            return 0

        result = 1
        try:
            ct = self.docker.create_container(
                image=self.cloner_image, user=str(self.docker_cmd_uid),
                command=['rm', '-rf', '/tmp/code'])
            self.docker.start(container=ct.get('Id'),
                              binds={destination: '/tmp'})
            result = self.docker.wait(container=ct.get('Id'))
            self.docker.remove_container(container=ct.get('Id'))
        except (errors.DockerException, errors.APIError) as e:
            clone_dir = '{}/code'.format(destination)
            LOG.error('Error in remove cloned repo %s, %s' %
                      (clone_dir, str(e)))

        return result

    def _clone_repo(self, repo_url, destination, logger, revision='master'):
        # Clone a repo with the constraints of disk and memory usage
        # Need to consider limiting network bandwidth as well.
        container_dest = '/tmp/code'
        if utils.is_git_sha(revision):
            clone_cmd = ('git clone {url} {dst} &&'
                         ' cd {dst} &&'
                         ' git checkout -B solum {rev} &&'
                         ' echo sha=$(git log -1 --pretty=%H)').format(
                url=repo_url, dst=container_dest, rev=revision)
        else:
            clone_cmd = ('git clone -b {branch} --depth 1 {url} {dst} &&'
                         ' cd {dst} &&'
                         ' echo sha=$(git log -1 --pretty=%H)').format(
                branch=revision, url=repo_url, dst=container_dest)

        timeout_clone = 'timeout --signal=SIGKILL {t} {clone}'.format(
            t=GIT_CLONE_TIMEOUT, clone=clone_cmd)

        dockerfile = ('FROM solum/cloner\n'
                      'RUN groupadd -f -g {gid} s-cloner-group\n'
                      'RUN useradd -s /bin/bash -u {uid} -g {gid} s-cloner\n'
                      'USER s-cloner\n'
                      'CMD {cmd}').format(uid=self.docker_cmd_uid,
                                          gid=cloner_gid,
                                          cmd=timeout_clone)

        ranid = ''.join(random.choice(string.digits) for _ in range(5))
        self.cloner_image = '{}-cloner-{}'.format(self.docker_cmd_uid, ranid)
        try:
            self._docker_build_with_retry(
                self.cloner_image, logger, pull=False,
                fileobj=io.BytesIO(dockerfile.encode('utf-8')))
            ct = self.docker.create_container(image=self.cloner_image,
                                              mem_limit=mem_limit,
                                              memswap_limit=-1)
        except (errors.DockerException, errors.APIError) as e:
            logger.log(logging.ERROR, 'Pre git clone stage failed.')
            LOG.error('Error in building/creating container for cloning,'
                      ' assembly: %s, %s' % (self.assembly.uuid, str(e)))
            return

        head_sha = None
        for i in range(MAX_GIT_CLONE_RETRY):
            # retry cloning
            try:
                self.docker.start(container=ct.get('Id'),
                                  binds={destination: '/tmp'})
                for line in self.docker.attach(container=ct.get('Id'),
                                               stream=True):
                    if line.startswith('sha='):
                        head_sha = line.replace('sha=', '').strip()
                    else:
                        logger.log(logging.INFO, line)
            except (errors.DockerException, errors.APIError) as e:
                logger.log(logging.ERROR, 'Got an error in cloning the repo,'
                                          ' max retry %s times. Repo: %s,'
                                          ' revision: %s' %
                           (MAX_GIT_CLONE_RETRY, repo_url, revision))
                LOG.error('Error in cloning. assembly: %s, repo: %s,'
                          ' rev: %s, %s' %
                          (self.assembly.uuid, repo_url, revision, str(e)))
            if head_sha:
                logger.log(logging.INFO, 'Finished cloning repo.')
                break
            elif i < MAX_GIT_CLONE_RETRY - 1:
                clone_dir = '{}/code'.format(destination)
                res = self._remove_cloned_repo(destination)
                if res != 0:
                    LOG.critical('critical: cannot remove dir %s,'
                                 ' disk may be full.' % clone_dir)
                time.sleep(3)

        try:
            self.docker.remove_container(container=ct.get('Id'))
        except (errors.DockerException, errors.APIError):
            pass

        return head_sha

    def _gen_docker_ignore(self, path, prefix=None):
        # Exclude .git from the docker build context
        content = '{}/.git'.format(prefix) if prefix else '.git'
        try:
            with open('{}/.dockerignore'.format(path), 'w') as f:
                f.write(content)
        except OSError:
            pass

    def _docker_build(self, tag, logger, timeout, limits, path=None,
                      dockerfile=None, fileobj=None, forcerm=True, quiet=True,
                      nocache=False, pull=True):
        success = 1
        try:
            for l in self.docker.build(path=path, dockerfile=dockerfile,
                                       fileobj=fileobj, tag=tag,
                                       timeout=timeout, forcerm=forcerm,
                                       quiet=quiet, nocache=nocache, pull=pull,
                                       container_limits=limits):
                try:
                    info = json.loads(l).get('stream', '')
                    if info:
                        if 'successfully built' in info.lower():
                            success = 0
                    else:
                        err = json.loads(l).get('errorDetail', '')
                        if err:
                            logger.log(logging.ERROR, err)
                except ValueError:
                    pass
        except req_exp.ReadTimeoutError:
            logger.log(logging.ERROR, 'docker build timed out, max value: %s' %
                       timeout)
        except (errors.DockerException, errors.APIError) as e:
            LOG.error('Error in building docker image %s, assembly: %s, %s' %
                      (tag, self.assembly.uuid, str(e)))
        return success

    def _docker_build_with_retry(self, tag, logger, path=None, dockerfile=None,
                                 fileobj=None, forcerm=True, quiet=True,
                                 limits=None, pull=True,
                                 timeout=build_timeout):
        limits = limits or {'memory': mem_limit, 'memswap': -1}

        result = self._docker_build(tag, logger, timeout, limits, path=path,
                                    dockerfile=dockerfile, fileobj=fileobj,
                                    forcerm=forcerm, quiet=quiet, pull=pull,
                                    nocache=False)
        if result == 0:
            return 0

        time.sleep(2)
        result = self._docker_build(tag, logger, timeout, limits, path=path,
                                    dockerfile=dockerfile, fileobj=fileobj,
                                    forcerm=forcerm, quiet=quiet, pull=pull,
                                    nocache=True)

        return result

    @utils.retry
    def _docker_save(self, image, output):
        result = 1
        try:
            lp = self.docker.get_image(image)
            with open(output, 'w') as f:
                f.write(lp.data)
            result = 0
        except (OSError, errors.DockerException, errors.APIError) as e:
            LOG.error('Error saving docker image, %s' % str(e))
        return result

    @utils.retry
    def _docker_load(self, path):
        result = 1
        try:
            with open(path, 'rb') as f:
                self.docker.load_image(f)
            result = 0
        except (OSError, errors.DockerException, errors.APIError) as e:
            LOG.error('Error in loading docker image, %s' % str(e))
        return result

    def _persist_to_backend(self, local_file, swift_container, swift_obj,
                            logger):
        loc = None
        if (self.image_storage == 'glance' or
                self.image_storage == 'docker_registry'):
            return loc
        elif self.image_storage == 'swift':
            swift = solum_swiftclient.SwiftClient(self.context)
            try:
                swift.upload(local_file, swift_container, swift_obj)
                loc = swift_obj
            except exc.InvalidObjectSizeError:
                logger.log(logging.INFO, 'Image with size exceeding 5GB'
                                         ' is not supported')
            except swiftexp.ClientException as e:
                LOG.error('Error in persisting artifact to swift, %s' % str(e))
            return loc

    def unittest_app(self, *args):
        """Interface to implement in derived class."""
        pass

    def build_app(self, *args):
        """Interface to implement in derived class."""
        pass



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Add user logs table

Revision ID: 450600086a09
Revises: 498adc6185ae
Create Date: 2014-09-29 20:43:55.544682

"""
from alembic import op
from oslo_utils import timeutils
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '450600086a09'
down_revision = '498adc6185ae'


def upgrade():
    op.create_table(
        'userlogs',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow,
                  nullable=False),
        sa.Column('updated_at', sa.DateTime, default=timeutils.utcnow,
                  nullable=False),
        sa.Column('assembly_uuid', sa.String(36), nullable=False),
        sa.Column('location', sa.String(255), nullable=False),
        sa.Column('strategy', sa.String(255), nullable=False),
        sa.Column('strategy_info', sa.String(1024), nullable=False),
        )


def downgrade():
    op.drop_table('userlogs')
